@@ -72,7 +72,7 @@ mktime_from_string (char *date_s,
 
 	fields = g_strsplit (time_s, ":", 3);
 	if (fields[0] != NULL) {
-		tm.tm_hour = atoi (fields[0]) - 1;
+		tm.tm_hour = atoi (fields[0]);
 		if (fields[1] != NULL) {
 			tm.tm_min  = atoi (fields[1]);
 			if (fields[2] != NULL)
@@ -85,118 +85,6 @@ mktime_from_string (char *date_s,
 }
 
 
-static gboolean
-_match_pattern (const char *line, 
-		const char *pattern)
-{
-	const char *l = line, *p = pattern;
-
-	for (; (*p != 0) && (*l != 0); p++, l++) {
-		if (*p != '%') {
-			if (*p != *l)
-				return FALSE;
-		} else {
-			p++;
-			switch (*p) {
-			case 'a':
-				break;
-			case 'n':
-				if (!isdigit (*l)) 
-					return FALSE;
-				break;
-			case 'c':
-				if (!isalpha (*l)) 
-					return FALSE;
-				break;
-			default:
-				return FALSE;
-			}
-		}
-	}
-	
-	return (*p == 0);
-}
-
-
-static int
-get_index_from_pattern (const char *line, 
-			const char *pattern)
-{
-	int         line_l, pattern_l;
-	const char *l;
-
-	line_l = strlen (line);
-	pattern_l = strlen (pattern);
-
-	if ((pattern_l == 0) || (line_l == 0))
-		return -1;
-
-	for (l = line; *l != 0; l++) 
-		if (_match_pattern (l, pattern))
-			return (l - line);
-	
-	return -1;
-}
-
-
-static char*
-get_next_field (const char *line,
-		int         start_from,
-		int         field_n)
-{
-	const char *f_start, *f_end;
-	
-	line = line + start_from;
-
-	f_start = line;
-	while ((*f_start == ' ') && (*f_start != *line))
-		f_start++;
-	f_end = f_start;
-
-	while ((field_n > 0) && (*f_end != 0)) {
-		if (*f_end == ' ') {
-			field_n--;
-			if (field_n != 0) {
-				while ((*f_end == ' ') && (*f_end != *line))
-					f_end++;
-				f_start = f_end;
-			}
-		} else
-			f_end++;
-	}
-	
-	return g_strndup (f_start, f_end - f_start);
-}
-
-
-static char*
-get_prev_field (const char *line,
-		int         start_from,
-		int         field_n)
-{
-	const char *f_start, *f_end;
-
-	f_start = line + start_from - 1;
-	while ((*f_start == ' ') && (*f_start != *line))
-		f_start--;
-	f_end = f_start;
-
-	while ((field_n > 0) && (*f_start != *line)) {
-		if (*f_start == ' ') {
-			field_n--;
-			if (field_n != 0) {
-				while ((*f_start == ' ') && (*f_start != *line))
-					f_start--;
-				f_end = f_start;
-			}
-		} else
-			f_start--;
-	}
-
-	return g_strndup (f_start + 1, f_end - f_start);
-}
-
-
 static char*
 tar_get_last_field (const char *line,
 		    int         start_from,
@@ -241,14 +129,14 @@ process_line (char     *line,
 
 	fdata = file_data_new ();
 
-	date_idx = get_index_from_pattern (line, "%n%n%n%n-%n%n-%n%n %n%n:%n%n");
+	date_idx = file_list__get_index_from_pattern (line, "%n%n%n%n-%n%n-%n%n %n%n:%n%n");
 
-	field_size = get_prev_field (line, date_idx, 1);
+	field_size = file_list__get_prev_field (line, date_idx, 1);
 	fdata->size = atol (field_size);
 	g_free (field_size);
 
-	field_date = get_next_field (line, date_idx, 1);
-	field_time = get_next_field (line, date_idx, 2);
+	field_date = file_list__get_next_field (line, date_idx, 1);
+	field_time = file_list__get_next_field (line, date_idx, 2);
 	fdata->modified = mktime_from_string (field_date, field_time);
 	g_free (field_date);
 	g_free (field_time);
@@ -324,13 +212,25 @@ add_compress_arg (FRCommand *comm)
 static void
 begin_tar_command (FRCommand *comm)
 {
-	char *path = NULL;
+	char       *path = NULL;
+	const char *prev_path = NULL;
+	char       *temp = NULL;
+
+	/* In solaris gtar is present under /usr/sfw/bin */
+
+	prev_path = g_getenv ("PATH");
+	temp = g_strdup_printf ("PATH=%s:%s", prev_path, "/usr/sfw/bin");
+	putenv (temp);
 
 	path = g_find_program_in_path ("gtar");
 	if (path != NULL)
-		fr_process_begin_command (comm->process, "gtar");
+		fr_process_begin_command (comm->process, path);
 	else
 		fr_process_begin_command (comm->process, "tar");
+
+	temp = g_strdup_printf ("PATH=%s", prev_path);
+	putenv (temp);
+
 	g_free (path);
 }
 
@@ -405,7 +305,7 @@ fr_command_tar_add (FRCommand     *comm,
 	fr_process_add_arg (comm->process, "-v");
 
 	if (base_dir != NULL) {
-		gchar *e_base_dir = shell_escape (base_dir);
+		char *e_base_dir = shell_escape (base_dir);
 		fr_process_add_arg (comm->process, "-C");
 		fr_process_add_arg (comm->process, e_base_dir);
 		g_free (e_base_dir);
@@ -484,7 +384,7 @@ fr_command_tar_extract (FRCommand  *comm,
 {
 	GList *scan;
 
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+	fr_process_set_out_line_func (comm->process, 
 				      process_line__extract,
 				      comm);
 
@@ -846,6 +746,29 @@ fr_command_tar_uncompress (FRCommand *comm)
 }
 
 
+char *
+fr_command_tar_escape (FRCommand     *comm,
+		       const char    *str)
+{
+	char *estr;
+	char *estr2;
+
+        estr = escape_str (str, "?*\\'& !|()@#:;");
+        estr2 = escape_str_common (estr, "[]", '[', ']');
+	g_free (estr);
+
+	/*
+	  if ((strcmp (estr2, str) != 0) || strchrs (str, "& !()|@#:;")) {
+	  char *estr3 = g_strconcat ("'", estr2, "'", NULL);
+	  g_free (estr2);
+	  estr2 = estr3;
+	  }
+	*/
+
+	return estr2;
+}
+
+
 static void 
 fr_command_tar_class_init (FRCommandTarClass *class)
 {
@@ -864,6 +787,8 @@ fr_command_tar_class_init (FRCommandTarClass *class)
 
 	afc->recompress   = fr_command_tar_recompress;
 	afc->uncompress   = fr_command_tar_uncompress;
+
+	afc->escape       = fr_command_tar_escape;
 }
 
  
